#ifndef BALL_HPP
#define BALL_HPP

#ifdef USE_OPEN_GL
#ifdef __APPLE__
#include <GLUT/glut.h>
#else
#include <GL/glut.h>
#endif
#endif

#include <iostream>
using std::ostream;

#include <iomanip>
using std::setw;

#include <fstream>
using std::ofstream;

class ball
{
public:
	// member data
	double mass;
	double radius;
	double x;
	double y;

	// constructor
	ball() : mass(0), radius(0), x(0), y(0) {}
	ball(double m_, double r_, double x_, double y_) : mass(m_), radius(r_), x(x_), y(y_) {}

#ifdef USE_OPEN_GL
	void glDraw(double L) const
	{
		glBegin(GL_LINE_LOOP); 
			glColor3f(1.0, 0.5, 0.25); 
			for(int j = 0; j < 360; ++j)
				glVertex2f(2.0*(x+radius*cos(j*M_PI/180.0))/L-1.0,2.0*(y+radius*sin(j*M_PI/180.0))/L-1.0);
		glEnd();
		glFlush();
	}
#endif
	
	void psDraw(ostream &fout) const
	{
		fout << "newpath" << endl;
		fout << x << " " << y << " " << radius << " 0 360 arc" << endl;
		fout << "closepath" << endl;
		fout << "stroke" << endl;
	}
	
	// a friend function does not belong to the class but is affiliated with it
	friend ostream& operator<<(ostream& os, const ball &b)
	{
		os.precision(8);
		os << setw(20) << b.mass
		    << setw(20) << b.radius
		    << setw(20) << b.x
		    << setw(20) << b.y;
		return os;
	}
};

#endif // BALL_HPP
