#include <cassert>

#include <cstdlib>
using std::atol;
using std::atof;

#include <iostream>
using std::cerr;
using std::cout;
using std::endl;

#include <iomanip>
using std::setw;

#include <fstream>
using std::ofstream;

#include "updates.h"
#include "mtrand.hpp"

int main(int argc, char* argv[])
{
	if (argc != 3 and argc != 4)
	{
		cerr << "usage: packing_openGL (linear size) (distributional disorder) [seed]" << endl;
		exit(1);
	}
	L = atof(argv[1]);
	const double D = atof(argv[2]);
	if (L < 0.0 or D < 0.0)
	{
		cerr << "Error: arguments must be postitive" << endl;
		exit(1);
	}
	if (argc == 4)
	{
		const mtrand::uint32 S = atol(argv[3]);
		Rand.seed(S);
	}
	initialize(D);
	
	const double E1 = 1.0/(2*D+1);
	T = 10.0*E1; // (!) start with T set to some multiple of E1
    double Emin = E;
	
	// (!) loop until T falls below some lower bound:
	// (!)    make calls to update()
	// (!)    keep Emin current
	// (!)    reduce T
		
	ofstream fout("energy.dat",std::ios::app);
	fout << setw(20) << D << setw(20) << Emin/gravel.size() << setw(20) << Emin/totalM << endl;
	fout.close();
	
	fout.open("config.eps");
	generate_postscript(fout);
	fout.close();
	
	return 0;
}
