#include <cstddef>
using std::size_t;

#include <cstdlib>
using std::exit;

#include <cassert>

#include <cstring>
using std::strlen;

#ifdef __APPLE__
#include <GLUT/glut.h>
#else
#include <GL/glut.h>
#endif

#include <iostream>
using std::ostream;
using std::cerr;
using std::cout;
using std::endl;
using std::fixed;

#include <fstream>
using std::ofstream;

#include <sstream>
using std::stringstream;

#include <algorithm>
using std::min;

#include <cmath>
using std::log;
using std::sqrt;

#include "updates.h"
#include "mtrand.hpp"

double E0;

void glutPrint(float x, float y, const char* text, float r, float g, float b, float a) 
{ 
    if(!text || !strlen(text)) return; 
    bool blending = false; 
    if(glIsEnabled(GL_BLEND)) blending = true; 
    glEnable(GL_BLEND); 
    glColor4f(r,g,b,a); 
    glRasterPos2f(x,y); 
    while (*text) { 
        glutBitmapCharacter(GLUT_BITMAP_9_BY_15, *text); 
        text++; 
    } 
    if(!blending) glDisable(GL_BLEND); 
}

void display()
{
	glClear(GL_COLOR_BUFFER_BIT);
	for (vector<ball>::const_iterator step = gravel.begin(); step != gravel.end(); ++step)
		step->glDraw(1.11111*L);

	glBegin(GL_POLYGON);
	glColor3f(0.9,0.05,0.05);
	const double height = 1.9*min(1.0,T/E0)-0.95;
	glVertex2f(0.85,height);
	glVertex2f(0.95,height);
	glVertex2f(0.95,-0.95);
	glVertex2f(0.85,-0.95);
	glEnd();
	glBegin(GL_LINE_LOOP);
	glColor3f(1.0,1.0,1.0);
	glVertex2f(0.85,0.95);
	glVertex2f(0.95,0.95);
	glVertex2f(0.95,-0.95);
	glVertex2f(0.85,-0.95);
	glEnd();
	glBegin(GL_LINES);
	glColor3f(1.0,1.0,1.0);
	glVertex2f(0.8,-1.0);
	glVertex2f(0.8,1.0);
	glEnd();
	
	stringstream s;
	s.precision(3);
	s << "temperature = " << fixed << T << "; energy = " << fixed << E;
	glutPrint(-0.88f, -0.88f,s.str().c_str(), 1.0f, 1.0f, 1.0f, 0.5f);
	glFlush();
	glutSwapBuffers();
}

int startx, starty;
bool moving;
void mouse(int button, int state, int x, int y)
{
	if (button == GLUT_LEFT_BUTTON) 
	{
		if (state == GLUT_DOWN) 
		{
			moving = true;
			startx = x;
			starty = y;
		}
		if (state == GLUT_UP) 
		{
			moving = false;
		}
	}
}

void motion(int x, int y)
{
	if (moving) 
		T += 0.00125*E0*GLfloat(starty - y);
	if (T > E0) T = E0;
	else if (T < 0.000001*E0) T = 0.000001*E0;
	startx = x;
	starty = y;		
}

unsigned int filenum = 0;
void keyboard(unsigned char key, int x, int y)
{
	if (key == 'p')
	{
		stringstream s;
		s << "config" << (++filenum) << ".eps";
		ofstream fout(s.str().c_str());
		generate_postscript(fout);
		fout.close();
		cout << "Writing configuration to file " << s.str() << endl;
	}
	else if (key == 'q') exit(0);
}

const int delay = 0; // milliseconds
void animate(int)
{
	for (size_t n = 0; n < gravel.size(); ++n)
		update();
	glutPostRedisplay();
	glutTimerFunc(delay,animate,0);
}

int main(int argc, char **argv)
{
	if (argc != 3 and argc != 4)
	{
		cerr << "usage: packing_openGL (linear size) (distributional disorder) [seed]" << endl;
		exit(1);
	}
	L = atof(argv[1]);
	const double D = atof(argv[2]);
	if (L < 0.0 or D < 0.0)
	{
		cerr << "Error: arguments must be postitive" << endl;
		exit(1);
	}
	if (argc == 4)
	{
		const mtrand::uint32 S = atol(argv[3]);
		Rand.seed(S);
	}
	initialize(D);
	E0=10.0/(2*D+1);
	T=0.01*E0;
	glutInit(&argc,argv);
	glutInitDisplayMode (GLUT_RGB | GLUT_DOUBLE);
	glutInitWindowSize (800, 800);
	glutInitWindowPosition (100, 100);
	glutCreateWindow("Gravel packing");
	glutDisplayFunc(display);
	glutMouseFunc(mouse);
	glutMotionFunc(motion);	
	glutKeyboardFunc(keyboard);
	glutTimerFunc(delay,animate,0);
	glutMainLoop();
}
