
#include <cassert>

#include <cstddef>
using std::size_t;

#include <cmath>
using std::fabs;
using std::pow;

#include <iostream>
using std::cerr;
using std::cout;
using std::endl;

#include <vector>
using std::vector;

#include <numeric>
using std::accumulate;

#include <algorithm>
using std::sort;
using std::min_element;

#include "mtrand.hpp"
mtrand Rand;

#include "ball.hpp"

vector<ball> gravel;

double L, T = 0.0, E = 0.0, totalM = 0.0;

const double eps = 1.0E-8;

void initialize(double D)
{
	gravel.resize(0);
	E = totalM = 0.0;
	bool offset = false;
	for (double y = 1.01; totalM < 0.1*L*L; y += 1.77)
	{
		for (double x = 1.01; x+(offset ? 1.005 : 0.0) < L-1.01; x += 2.05)
		{
			gravel.push_back(ball(1.0,1.0,x+(offset ? 1.005 : 0.0),y));
			totalM += 1.0;
			E += y;
		}
		offset = !offset;
	}
}

void generate_postscript(ostream &fout)
{
	fout << "%!PS-Adobe-3.0 EPSF-3.0" << endl;
    fout << "%%BoundingBox: 0 0 612 612" << endl;
    const double s = 612.0/(L+2);
    fout << s << " " << s << " scale" << endl;
    fout << "1 1 translate" << endl;
	fout << "0.05 setlinewidth" << endl;
	fout << "0 setgray" << endl;
	for (vector<ball>::const_iterator step = gravel.begin(); step != gravel.end(); ++step)
		step->psDraw(fout);
}

void update(void)
{
	static vector<double> collision_times_plus;
	static vector<double> collision_times_minus;
	collision_times_plus.resize(0);
	collision_times_minus.resize(0);
	const size_t i = Rand.randIntExc(gravel.size());
	const double theta = M_PI*Rand();
	const double costheta = cos(theta);
	const double sintheta = sin(theta);
	assert(sintheta >= 0.0);
	for (size_t j = 0; j < gravel.size(); ++j)
	{
		if (i != j)
		{
			const double dx = gravel[i].x - gravel[j].x;
			const double dy = gravel[i].y - gravel[j].y;
			const double R = gravel[i].radius + gravel[j].radius;
			const double b = dx*costheta + dy*sintheta;
			const double c2 = b*b+R*R-dx*dx-dy*dy;
			if (c2 >= 0.0)
			{
				const double c = sqrt(c2);
				const double t1 = -b + c;
				const double t2 = -b - c;
				if (t1 >= 0.0) 
					collision_times_plus.push_back(t1);
				else
					collision_times_minus.push_back(t1);
				if (t2 >= 0.0) 
					collision_times_plus.push_back(t2);
				else
					collision_times_minus.push_back(t2);
			}
		}
	}
	
	const double tbottom = (gravel[i].radius - gravel[i].y)/sintheta;
	collision_times_minus.push_back(tbottom);

	assert(!collision_times_minus.empty());
	const bool upperlimit = !collision_times_plus.empty();
	const double dy2 = (upperlimit ? sintheta*(*min_element(collision_times_plus.begin(),collision_times_plus.end())) : 1.0);
	const double dy1 = sintheta*(*max_element(collision_times_minus.begin(),collision_times_minus.end()));
	assert(dy2 >= dy1);
	const double xi = Rand();
	const double Tm = T/gravel[i].mass;
	assert(Tm > eps);
	const double C = (upperlimit ? 1.0/( -dy1 + Tm*(1.0-exp(-dy2/Tm)) ) : 1.0/( -dy1 + Tm ) );
	const double dy = ( xi < -dy1*C ? xi/C + dy1 : -Tm*log(1-(xi/C + dy1)/Tm) );
	gravel[i].y += dy;
	gravel[i].x += dy*costheta/sintheta;
	E += gravel[i].mass*dy;
}
